from .hash import *
from .jtc64 import *
from secrets import randbelow as rb
import time



def getDiff(s1:str,s2:str): # s1-s2
    global nums,digit
    s3=''
    for i in range(0,len(s1)):
        s3=s3+digit[(16+nums[s1[i]]-nums[s2[i]])%16]
    return s3




def encrypt(text:str,key:str):
    if(text==''):
        raise InputError
    key=getHash(key)
    ran=getHash(getHash(text)+str(rb(2**32))+str(rb(2**32))+str(time.time())+str(rb(2**32))+str(rb(2**32)))[0:32]
    text=toHex(text)
    text=text+getHash(text)[0:16]
    l=len(text)
    n=0
    if(l%64==0):
        n=l//64
    else:
        n=l//64+1
    secret=''
    for i in range(0,n):
        secret=secret+getHash(str(i)+ran+key)
    secret=secret[0:l]
    text2=getSum(secret,text)
    text2=ran+text2
    text2=getHash(text2)[0:16]+text2+'0' # 最后的 0 是模式, 以后可能会加入新的加密模式, 便于兼容前面加密的结果
    return hexToJtc64(text2)
    
    




def decrypt(text:str,key:str):
    key=getHash(key)
    if(text==''):
        raise InvalidCiphertextError
    try:
        text=jtc64ToHex(text)
    except:
        raise InvalidCiphertextError
    if(text[-1]!='0'):
        raise VersionNotSupportError
    text=text[:-1]
    if(len(text)<=64):
        raise InvalidCiphertextError
    check1=text[0:16]
    if(getHash(text[16:])[0:16]!=check1):
        raise InvalidCiphertextError
    ran=text[16:48]
    text=text[48:]
    l=len(text)
    n=0
    if(l%64==0):
        n=l//64
    else:
        n=l//64+1
    secret=''
    for i in range(0,n):
        secret=secret+getHash(str(i)+ran+key)
    secret=secret[0:l]
    text2=getDiff(text,secret)
    if(getHash(text2[0:-16])[0:16]!=text2[-16:]):
        raise PasswordWrongError
    return hexToStr(text2[0:-16])





'''
jtc64 类似于 base64, 但是在处理多余位数时需要的位数更少, 同时使用的符号更易辨别
base64 是每6位变4位, jtc64是每3位变2位; base64 多2位和多4位都需要额外的4个字符, jtc64多1位只需1个字符, 多2位只需2个字符
正常情况使用 0~9, a~z, A~Z, @, #
在多2位的情况下要使用 $, %, &, =
'''

from .exceptions import *
import codecs
chars='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@#'
dic1={}
for i in range(0,64):
    dic1[chars[i]]=i
hexs='0123456789abcdef'
dic2={}
for i in range(0,16):
    dic2[hexs[i]]=i
syms='$%&='
dic3={'$':0,'%':1,'&':2,'=':3}




def hexToStr(text:str):
    return codecs.decode(text,'hex').decode('utf-8')

def toHex(text:str):
    text=text.encode('utf-8')
    return codecs.encode(text,'hex').decode('utf-8')



def hexToJtc64(text:str):
    try:
        l=len(text)
        n=l//3
        r=l-n*3
        text2=''
        for i in range(0,n):
            num=dic2[text[3*i]]*256+dic2[text[3*i+1]]*16+dic2[text[3*i+2]]
            s1=num//64
            s2=num%64
            text2=text2+chars[s1]+chars[s2]
        if(r==0):
            return text2
        if(r==1):
            return text2+text[-1]
        num2=dic2[text[3*n]]*16+dic2[text[3*n+1]]
        text2=text2+chars[num2//4]+syms[num2%4]
        return text2
    except:
        raise NotHexError



def strToJtc64(text:str):
    text=toHex(text)
    return hexToJtc64(text)


def jtc64ToHex(text:str):
    try:
        if(text==''):
            return ''
        type=0
        main=text
        if(len(text)%2==1):
            type=1
            main=text[:-1]
        if(text[-1] in ['$','%','&','=']):
            type=2
            main=text[:-2]
        n=len(main)//2
        text2=''
        for i in range(0,n):
            num=dic1[text[2*i]]*64+dic1[text[2*i+1]]
            text2=text2+hexs[num//256]+hexs[(num%256)//16]+hexs[num%16]
        if(type==0):
            return text2
        if(type==1):
            return text2+text[-1]
        num2=dic1[text[2*n]]*4+dic3[text[2*n+1]]
        text2=text2+hexs[num2//16]+hexs[num2%16]
        return text2
    except:
        raise NotJtc64Error


def jtc64ToStr(text:str):
    return hexToStr(jtc64ToHex(text))




from .exceptions import NotHexError,NotJtc64Error,InputError,InvalidCiphertextError,VersionNotSupportError,PasswordWrongError
from .hash import getHash
from .crypt import encrypt,decrypt
from .jtc64 import hexToStr,toHex,hexToJtc64,strToJtc64,jtc64ToHex,jtc64ToStr


__all__=[
    'NotHexError','NotJtc64Error','InputError','InvalidCiphertextError','VersionNotSupportError','PasswordWrongError',
    'getHash',
    'encrypt','decrypt',
    'hexToStr','toHex','hexToJtc64','strToJtc64','jtc64ToHex','jtc64ToStr'
]


__version__='1.0.1'
__license__='GPL-2.0 License'
__author__='Tiancheng Jiao'
__github__='https://github.com/jtc1246/mySecrets'
__author_email__='jtc1246@outlook.com'
__description__='Very secure hash and symmetrical encryption'


class NotHexError(Exception):
    def __init__(self):
        Exception.__init__(self,'Input is not in hex format.')


class NotJtc64Error(Exception):
    def __init__(self):
        Exception.__init__(self,'Input is not in jtc64 format.')

class InputError(Exception):
    def __init__(self):
        Exception.__init__(self,'The length of text must be larger than 0.')

class InvalidCiphertextError(Exception):
    def __init__(self):
        Exception.__init__(self,'Invalid Ciphertext. Please check if there is something wrong during transportation.')

class VersionNotSupportError(Exception):
    def __init__(self):
        Exception.__init__(self,'This version cannot decrypt the data encrypted in a new version. Please update this package.')

class PasswordWrongError(Exception):
    def __init__(self):
        Exception.__init__(self,'Password is not correct.')





from setuptools import setup

setup(
    name='mySecrets',
    version='1.0.1',
    author='Tiancheng Jiao',
    author_email='jtc1246@outlook.com',
    url='https://github.com/jtc1246/mySecrets',
    description='Very secure hash and symmetrical encryption',
    packages=['mySecrets'],
    python_requires='>=3',
    platforms=["all"],
    license='GPL-2.0 License',
    entry_points={
        'console_scripts': [
            'getHash=mySecrets:getHash',
            'encrypt=mySecrets:encrypt',
            'decrypt=mySecrets:decrypt',
            'hexToStr=mySecrets:hexToStr',
            'toHex=mySecrets:toHex',
            'hexToJtc64=mySecrets:hexToJtc64',
            'strToJtc64=mySecrets:strToJtc64',
            'jtc64ToHex=mySecrets:jtc64ToHex',
            'jtc64ToStr=mySecrets:jtc64ToStr',
            'NotHexError=mySecrets:NotHexError',
            'NotJtc64Error=mySecrets:NotJtc64Error',
            'InputError=mySecrets:InputError',
            'InvalidCiphertextError=mySecrets:InvalidCiphertextError',
            'VersionNotSupportError=mySecrets:VersionNotSupportError',
            'PasswordWrongError=mySecrets:PasswordWrongError'
        ]
    }
)

